@@ -1,111 +1,165 @@
-"""Tests of various paths through yakonfig.Configurable.
+'''Tests of various paths through yakonfig.Configurable.
 
------
+.. This software is released under an MIT/X11 open source license.
+   Copyright 2014-2015 Diffeo, Inc.
 
-This software is released under an MIT/X11 open source license.
-
-Copyright 2014 Diffeo, Inc.
-
-"""
+'''
 
 from __future__ import absolute_import
 
 import argparse
 
 import pytest
+from six import iterkeys
 
 import yakonfig
 import yakonfig.tests.configurable_module
 import yakonfig.toplevel
 
+
 class ConfigurableSubclass(yakonfig.Configurable):
     @property
-    def config_name(self): return 'configurable'
+    def config_name(self):
+        return 'configurable'
+
     @property
-    def default_config(self): return { 'type': 'object' }
+    def default_config(self):
+        return {'type': 'object'}
+
 
 class ConfigurableLike(object):
     config_name = 'configurable'
-    default_config = { 'type': 'class' }
-    runtime_keys = { 'key': 'key' }
+    default_config = {'type': 'class'}
+    runtime_keys = {'key': 'key'}
+
 
 class ConfigurableArgs(yakonfig.Configurable):
     @property
-    def config_name(self): return 'config'
+    def config_name(self):
+        return 'config'
+
     @property
-    def default_config(self): return {'k': 'k'}
+    def default_config(self):
+        return {'k': 'k'}
+
     def add_arguments(self, parser):
         parser.add_argument('-k', '--key', metavar='VALUE')
+
     @property
-    def runtime_keys(self): return {'key': 'k'}
+    def runtime_keys(self):
+        return {'key': 'k'}
+
     def check_config(self, config, name=''):
         if len(config['k']) != 1:
-            raise yakonfig.ConfigurationError("{} 'k' wrong length"
+            raise yakonfig.ConfigurationError("{0} 'k' wrong length"
                                               .format(name))
 
+
 class ConfigurableBottom(object):
     config_name = 'bottom'
-    default_config = { 'zzz': '-32768' }
+    default_config = {'zzz': '-32768'}
+
     @staticmethod
     def add_arguments(parser):
         parser.add_argument('--zzz', '-z')
-    runtime_keys = { 'zzz': 'zzz' }
+
+    runtime_keys = {'zzz': 'zzz'}
+
 
 class ConfigurableTop(object):
     config_name = 'top'
-    default_config = { 'aaa': 'bbb' }
+    default_config = {'aaa': 'bbb'}
     sub_modules = [ConfigurableBottom]
+
     @staticmethod
     def add_arguments(parser):
         parser.add_argument('--aaa', '-a')
-    runtime_keys = { 'aaa': 'aaa' }
+
+    runtime_keys = {'aaa': 'aaa'}
+
 
 class ConfigurableAlmostTop(object):
     config_name = 'top'
-    default_config = { 'aaa': 'bbb' }
+    default_config = {'aaa': 'bbb'}
+
     @staticmethod
     def add_arguments(parser):
         parser.add_argument('--aaa', '-a')
-    runtime_keys = { 'aaa': 'aaa' }
+
+    runtime_keys = {'aaa': 'aaa'}
+
     @staticmethod
     def replace_config(config, name):
         return ConfigurableTop
 
+
 class ConfigurableLikeTop(object):
     config_name = 'top'
-    default_config = { 'aaa': 'bbb' }
+    default_config = {'aaa': 'bbb'}
+
     @staticmethod
     def add_arguments(parser):
         parser.add_argument('--aaa', '-a')
-    runtime_keys = { 'aaa': 'aaa' }
+
+    runtime_keys = {'aaa': 'aaa'}
+
     @staticmethod
     def replace_config(config, name):
-        return yakonfig.NewSubModules(ConfigurableLikeTop, [ConfigurableBottom])
+        return yakonfig.NewSubModules(ConfigurableLikeTop,
+                                      [ConfigurableBottom])
 
-class Dependent:
+
+class Dependent(object):
     config_name = 'dependent'
+
     @staticmethod
     def check_config(config, name):
         yakonfig.check_toplevel_config(ConfigurableArgs(), name)
 
-class Normalized:
+
+class Normalized(object):
     config_name = 'normalized'
-    default_config = { 'k': 'value' }
-    runtime_keys = { 'k': 'k' }
+    default_config = {'k': 'value'}
+    runtime_keys = {'k': 'k'}
+
     @staticmethod
     def normalize_config(config):
         config['k'] = config['k'][0]
 
+
+class Discovers(object):
+    config_name = 'discovers'
+    default_config = {'a': 'one', 'c': 'three'}
+    runtime_keys = {'a': 'a', 'b': 'b', 'c': 'c'}
+
+    @staticmethod
+    def discover_config(config, name):
+        if 'a' not in config:
+            config['a'] = 'foo'
+        if 'b' not in config:
+            config['b'] = 'bar'
+
+
+class NoneDefault(object):
+    config_name = 'none_default'
+    default_config = {'none': None}
+
+
 @pytest.fixture(params=['object', 'class', 'module'])
 def configurable_type(request):
     return request.param
 
+
 @pytest.fixture
 def a_configurable(configurable_type):
-    if configurable_type == 'object': return ConfigurableSubclass()
-    if configurable_type == 'class': return ConfigurableLike
-    if configurable_type == 'module': return yakonfig.tests.configurable_module
-    raise RuntimeError(t)
+    if configurable_type == 'object':
+        return ConfigurableSubclass()
+    if configurable_type == 'class':
+        return ConfigurableLike
+    if configurable_type == 'module':
+        return yakonfig.tests.configurable_module
+    raise KeyError(configurable_type)
+
 
 @pytest.yield_fixture
 def global_yakonfig(a_configurable):
@@ -113,63 +167,71 @@ def global_yakonfig(a_configurable):
     yield yakonfig.get_global_config()
     yakonfig.clear_global_config()
 
+
 def test_assemble_default_config(a_configurable, configurable_type):
     c = yakonfig.toplevel.assemble_default_config([a_configurable])
-    assert sorted(c.iterkeys()) == ['configurable']
+    assert sorted(iterkeys(c)) == ['configurable']
     cc = c['configurable']
-    assert sorted(cc.iterkeys()) == ['type']
+    assert sorted(iterkeys(cc)) == ['type']
     assert cc['type'] == configurable_type
     assert cc.get('key') is None
 
+
 def test_assemble_minimal():
     class MinimallyConfigurable(object):
         config_name = 'minimal'
     c = yakonfig.toplevel.assemble_default_config([MinimallyConfigurable])
-    assert sorted(c.iterkeys()) == ['minimal']
+    assert sorted(iterkeys(c)) == ['minimal']
     assert c['minimal'] == {}
 
+
 def test_assemble_broken():
     class NotReallyConfigurable(object):
         pass
     with pytest.raises(yakonfig.ProgrammerError):
         yakonfig.toplevel.assemble_default_config([NotReallyConfigurable])
 
+
 def test_assemble_two():
     c = yakonfig.toplevel.assemble_default_config([ConfigurableArgs(),
                                                    ConfigurableSubclass()])
-    assert sorted(c.iterkeys()) == ['config', 'configurable']
+    assert sorted(iterkeys(c)) == ['config', 'configurable']
     assert c['config'] == {'k': 'k'}
     assert c['configurable'] == {'type': 'object'}
 
+
 def test_duplicates():
     with pytest.raises(yakonfig.ProgrammerError):
         yakonfig.toplevel.assemble_default_config([ConfigurableLike,
                                                    ConfigurableSubclass()])
 
+
 def test_config_type(global_yakonfig, configurable_type):
     c = global_yakonfig
-    assert sorted(c.iterkeys()) == ['configurable']
+    assert sorted(iterkeys(c)) == ['configurable']
     cc = c['configurable']
-    assert sorted(cc.iterkeys()) == ['type']
+    assert sorted(iterkeys(cc)) == ['type']
     assert cc['type'] == configurable_type
     assert cc.get('key') is None
 
+
 def test_fill_in():
-    c = { 'configurable': {} }
+    c = {'configurable': {}}
     yakonfig.toplevel.fill_in_arguments(c, [ConfigurableLike],
-                                        { 'key': 'value' })
-    assert sorted(c.iterkeys()) == ['configurable']
+                                        {'key': 'value'})
+    assert sorted(iterkeys(c)) == ['configurable']
     cc = c['configurable']
-    assert sorted(cc.iterkeys()) == ['key']
+    assert sorted(iterkeys(cc)) == ['key']
     assert cc['key'] == 'value'
 
+
 def test_fill_in_replaces():
     c = { 'configurable': { 'key': 'old' } }
     yakonfig.toplevel.fill_in_arguments(c, [ConfigurableLike],
                                         { 'key': 'new' })
-    assert sorted(c.iterkeys()) == ['configurable']
+    assert sorted(iterkeys(c)) == ['configurable']
     cc = c['configurable']
-    assert sorted(cc.iterkeys()) == ['key']
+    assert sorted(iterkeys(cc)) == ['key']
     assert cc['key'] == 'new'
 
 def test_fill_in_object():
@@ -178,9 +240,9 @@ def test_fill_in_object():
             self.key = 'value'
     c = { 'configurable': {} }
     yakonfig.toplevel.fill_in_arguments(c, [ConfigurableLike], Params())
-    assert sorted(c.iterkeys()) == ['configurable']
+    assert sorted(iterkeys(c)) == ['configurable']
     cc = c['configurable']
-    assert sorted(cc.iterkeys()) == ['key']
+    assert sorted(iterkeys(cc)) == ['key']
     assert cc['key'] == 'value'    
 
 def test_default_file(request):
@@ -189,9 +251,9 @@ def test_default_file(request):
         filename=str(request.fspath.dirpath('argconfig.yaml')))
     try:
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['config']
+        assert sorted(iterkeys(c)) == ['config']
         cc = c['config']
-        assert sorted(cc.iterkeys()) == ['k']
+        assert sorted(iterkeys(cc)) == ['k']
         assert cc['k'] == 'x'
     finally:
         yakonfig.clear_global_config()
@@ -200,9 +262,9 @@ def test_fill_in_kvp():
     yakonfig.set_default_config([ConfigurableLike], { 'key': 'value' })
     try:
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['configurable']
+        assert sorted(iterkeys(c)) == ['configurable']
         cc = c['configurable']
-        assert sorted(cc.iterkeys()) == ['key', 'type']
+        assert sorted(iterkeys(cc)) == ['key', 'type']
         assert cc['type'] == 'class'
         assert cc['key'] == 'value'
     finally:
@@ -216,9 +278,9 @@ configurable:
     yakonfig.set_default_config([ConfigurableLike], yaml=the_yaml)
     try:
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['configurable']
+        assert sorted(iterkeys(c)) == ['configurable']
         cc = c['configurable']
-        assert sorted(cc.iterkeys()) == ['key', 'type']
+        assert sorted(iterkeys(cc)) == ['key', 'type']
         assert cc['type'] == 'class'
         assert cc['key'] == 'yaml'
     finally:
@@ -235,9 +297,9 @@ configurable:
                                 yaml=the_yaml)
     try:
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['configurable']
+        assert sorted(iterkeys(c)) == ['configurable']
         cc = c['configurable']
-        assert sorted(cc.iterkeys()) == ['key', 'type']
+        assert sorted(iterkeys(cc)) == ['key', 'type']
         assert cc['type'] == 'class'
         assert cc['key'] == 'value'
     finally:
@@ -249,9 +311,9 @@ def test_dont_validate():
                                 validate=False)
     try:
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['config']
+        assert sorted(iterkeys(c)) == ['config']
         cc = c['config']
-        assert sorted(cc.iterkeys()) == ['k']
+        assert sorted(iterkeys(cc)) == ['k']
         assert cc['k'] == 'longer than one char'
     finally:
         yakonfig.clear_global_config()
@@ -260,12 +322,12 @@ def test_two_level():
     yakonfig.set_default_config([ConfigurableTop])
     try:
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['top']
+        assert sorted(iterkeys(c)) == ['top']
         c = c['top']
-        assert sorted(c.iterkeys()) == ['aaa', 'bottom']
+        assert sorted(iterkeys(c)) == ['aaa', 'bottom']
         assert c['aaa'] == 'bbb'
         c = c['bottom']
-        assert sorted(c.iterkeys()) == ['zzz']
+        assert sorted(iterkeys(c)) == ['zzz']
         assert c['zzz'] == '-32768'
     finally:
         yakonfig.clear_global_config()
@@ -275,12 +337,12 @@ def test_two_level_args():
                                 { 'aaa': 'a', 'bbb': 'b', 'zzz': 'z' })
     try:
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['top']
+        assert sorted(iterkeys(c)) == ['top']
         c = c['top']
-        assert sorted(c.iterkeys()) == ['aaa', 'bottom']
+        assert sorted(iterkeys(c)) == ['aaa', 'bottom']
         assert c['aaa'] == 'a'
         c = c['bottom']
-        assert sorted(c.iterkeys()) == ['zzz']
+        assert sorted(iterkeys(c)) == ['zzz']
         assert c['zzz'] == 'z'
     finally:
         yakonfig.clear_global_config()
@@ -389,7 +451,7 @@ def test_check_dependent():
     # application so it shouldn't be checked
     with yakonfig.defaulted_config([ConfigurableArgs(), ConfigurableLike],
                                    {'key': 'value'}) as config:
-        assert sorted(config.iterkeys()) == ['config', 'configurable']
+        assert sorted(iterkeys(config)) == ['config', 'configurable']
         with pytest.raises(yakonfig.ConfigurationError):
             yakonfig.check_toplevel_config(ConfigurableArgs(), 'test')
         with pytest.raises(yakonfig.ConfigurationError):
@@ -399,7 +461,7 @@ def test_check_toplevel():
     '''check_config_toplevel() should work in check_config() implementation'''
     with yakonfig.defaulted_config([ConfigurableArgs(), Dependent],
                                    {'key': 'k'}) as config:
-        assert sorted(config.iterkeys()) == ['config', 'dependent']
+        assert sorted(iterkeys(config)) == ['config', 'dependent']
         assert config['config']['k'] == 'k'
     with pytest.raises(yakonfig.ConfigurationError):
         with yakonfig.defaulted_config([ConfigurableArgs(), Dependent],
@@ -413,7 +475,7 @@ def test_check_toplevel_args():
                         ['--key', 'k'])
     try:
         config = yakonfig.get_global_config()
-        assert sorted(config.iterkeys()) == ['config', 'dependent']
+        assert sorted(iterkeys(config)) == ['config', 'dependent']
         assert config['config']['k'] == 'k'
     finally:
         yakonfig.clear_global_config()
@@ -428,18 +490,18 @@ def test_proxy_two_level():
             [yakonfig.ProxyConfigurable(ConfigurableTop)],
             { 'aaa': 'a', 'bbb': 'b', 'zzz': 'z' }):
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['top']
+        assert sorted(iterkeys(c)) == ['top']
         c = c['top']
-        assert sorted(c.iterkeys()) == ['aaa', 'bottom']
+        assert sorted(iterkeys(c)) == ['aaa', 'bottom']
         assert c['aaa'] == 'a'
         c = c['bottom']
-        assert sorted(c.iterkeys()) == ['zzz']
+        assert sorted(iterkeys(c)) == ['zzz']
         assert c['zzz'] == 'z'
 
 def test_replaces_direct():
     with yakonfig.defaulted_config([ConfigurableAlmostTop]):
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['top']
+        assert sorted(iterkeys(c)) == ['top']
         c = c['top']
         assert 'bottom' in c
         c = c['bottom']
@@ -448,7 +510,7 @@ def test_replaces_direct():
 def test_replaces_proxy():
     with yakonfig.defaulted_config([ConfigurableLikeTop]):
         c = yakonfig.get_global_config()
-        assert sorted(c.iterkeys()) == ['top']
+        assert sorted(iterkeys(c)) == ['top']
         c = c['top']
         assert 'bottom' in c
         c = c['bottom']
@@ -465,3 +527,31 @@ normalized:
   k: zoom!
 '''):
         assert yakonfig.get_global_config('normalized')['k'] == 'z'
+
+
+def test_discovery():
+    with yakonfig.defaulted_config([Discovers]):
+        # discovered overwrites default
+        assert yakonfig.get_global_config('discovers', 'a') == 'foo'
+        # discovered provides missing value
+        assert yakonfig.get_global_config('discovers', 'b') == 'bar'
+        # undiscovered uses default value
+        assert yakonfig.get_global_config('discovers', 'c') == 'three'
+    with yakonfig.defaulted_config([Discovers], {'a': 'alpha'}):
+        # command-line value overrules discovery
+        assert yakonfig.get_global_config('discovers', 'a') == 'alpha'
+        assert yakonfig.get_global_config('discovers', 'b') == 'bar'
+        assert yakonfig.get_global_config('discovers', 'c') == 'three'
+
+
+def test_none_default():
+    with yakonfig.defaulted_config([NoneDefault]):
+        assert yakonfig.get_global_config('none_default', 'none') is None
+
+
+def test_none_default_rt():
+    with yakonfig.defaulted_config([NoneDefault]):
+        config = yakonfig.get_global_config()
+
+    with yakonfig.defaulted_config([NoneDefault], config=config):
+        assert yakonfig.get_global_config('none_default', 'none') is None
