@@ -1,43 +1,52 @@
-"""Top-level entry points to yakonfig.
+'''Top-level entry points to yakonfig.
 
 .. This software is released under an MIT/X11 open source license.
-   Copyright 2014 Diffeo, Inc.
+   Copyright 2014-2015 Diffeo, Inc.
 
-Purpose
-=======
+Most programs' `main()` functions will call yakonfig as::
 
-Most programs' `main()` functions will call yakonfig as:
-
->>> parser = argparse.ArgumentParser()
->>> yakonfig.parse_args(parser, [yakonfig, module, module...])
+    parser = argparse.ArgumentParser()
+    yakonfig.parse_args(parser, [yakonfig, module, module...])
 
 where the list of modules are top-level modules or other
 :class:`yakonfig.Configurable` objects the program uses.
 
-Test code and other things not driven by argparse can instead call
+Test code and other things not driven by argparse can instead call::
 
->>> yakonfig.set_default_config([yakonfig, module, module, ...])
+    yakonfig.set_default_config([yakonfig, module, module, ...])
 
-Module Contents
-===============
+or::
 
-"""
+    with yakofnig.defaulted_config([yakonfig, ...]):
+      ...
+
+.. autofunction:: parse_args
+.. autofunction:: set_default_config
+.. autofunction:: defaulted_config
+
+'''
 
 from __future__ import absolute_import
 import collections
 import contextlib
 import copy
-from cStringIO import StringIO
+import logging
 import sys
 
-import yaml
+from six import iteritems, StringIO
+import yaml as yaml_mod
 
-import yakonfig
-from yakonfig.merge import overlay_config, diff_config
-from yakonfig.yakonfig import _temporary_config
+from .exceptions import ConfigurationError, ProgrammerError
+from .merge import overlay_config, diff_config
+from .yakonfig import get_global_config, set_global_config, _temporary_config
 
 # These implement the Configurable interface for yakonfig proper!
 config_name = 'yakonfig'
+
+
+logger = logging.getLogger(__name__)
+
+
 def add_arguments(parser):
     '''Add command-line arguments for yakonfig proper.
 
@@ -54,7 +63,8 @@ def add_arguments(parser):
     parser.add_argument('--dump-config', metavar='WHAT', nargs='?',
                         help='dump out configuration then stop '
                         '(default, effective, full)')
-runtime_keys = { 'config': 'config' }
+runtime_keys = {'config': 'config'}
+
 
 def parse_args(parser, modules, args=None):
     """Set up global configuration for command-line tools.
@@ -87,22 +97,25 @@ def parse_args(parser, modules, args=None):
     collect_add_argparse(parser, modules)
     namespace = parser.parse_args(args)
     try:
-        yakonfig.set_default_config(modules, params=vars(namespace))
-        if getattr(namespace, 'dump_config', None):
+        do_dump_config = getattr(namespace, 'dump_config', None)
+        set_default_config(modules, params=vars(namespace),
+                           validate=not do_dump_config)
+        if do_dump_config:
             if namespace.dump_config == 'full':
-                to_dump = yakonfig.get_global_config()
+                to_dump = get_global_config()
             elif namespace.dump_config == 'default':
                 to_dump = assemble_default_config(modules)
-            else: # 'effective'
+            else:  # 'effective'
                 to_dump = diff_config(assemble_default_config(modules),
-                                      yakonfig.get_global_config())
-            yaml.dump(to_dump, sys.stdout)
+                                      get_global_config())
+            yaml_mod.dump(to_dump, sys.stdout)
             parser.exit()
-    except yakonfig.ConfigurationError, e:
+    except ConfigurationError as e:
         parser.error(e)
     return namespace
 
-def set_default_config(modules, params={}, yaml=None, filename=None,
+
+def set_default_config(modules, params=None, yaml=None, filename=None,
                        config=None, validate=True):
     """Set up global configuration for tests and noninteractive tools.
 
@@ -125,6 +138,8 @@ def set_default_config(modules, params={}, yaml=None, filename=None,
     :returntype: dict
 
     """
+    if params is None:
+        params = {}
 
     # Get the configuration from the file, or from params['config']
     file_config = {}
@@ -132,12 +147,11 @@ def set_default_config(modules, params={}, yaml=None, filename=None,
         if 'config' in params and params['config'] is not None:
             filename = params['config']
     if yaml is not None or filename is not None or config is not None:
-        import yaml as y
         if yaml is not None:
-            file_config = y.load(StringIO(yaml))
+            file_config = yaml_mod.load(StringIO(yaml))
         elif filename is not None:
             with open(filename, 'r') as f:
-                file_config = y.load(f)
+                file_config = yaml_mod.load(f)
         elif config is not None:
             file_config = config
 
@@ -148,7 +162,7 @@ def set_default_config(modules, params={}, yaml=None, filename=None,
     fill_in_arguments(base_config, modules, params)
     default_config = assemble_default_config(modules)
     base_config = overlay_config(default_config, base_config)
-    
+
     # Replace the modules list (accommodate external modules)
     def replace_module(config, m):
         name = getattr(m, 'config_name')
@@ -174,16 +188,17 @@ def set_default_config(modules, params={}, yaml=None, filename=None,
         checker = getattr(mod, 'check_config', None)
         if checker is not None:
             with _temporary_config():
-                yakonfig.set_global_config(base_config)
+                set_global_config(base_config)
                 checker(base_config[mod.config_name], mod.config_name)
 
     # All done, normalize and set the global configuration
     normalize_config(base_config, modules)
-    yakonfig.set_global_config(base_config)
+    set_global_config(base_config)
     return base_config
 
+
 @contextlib.contextmanager
-def defaulted_config(modules, params={}, yaml=None, filename=None,
+def defaulted_config(modules, params=None, yaml=None, filename=None,
                      config=None, validate=True):
     """Context manager version of :func:`set_default_config()`.
 
@@ -213,7 +228,8 @@ def defaulted_config(modules, params={}, yaml=None, filename=None,
     with _temporary_config():
         set_default_config(modules, params=params, yaml=yaml,
                            filename=filename, config=config, validate=validate)
-        yield yakonfig.get_global_config()
+        yield get_global_config()
+
 
 def check_toplevel_config(what, who):
     """Verify that some dependent configuration is present and correct.
@@ -233,15 +249,16 @@ def check_toplevel_config(what, who):
 
     """
     config_name = what.config_name
-    config = yakonfig.get_global_config()
+    config = get_global_config()
     if config_name not in config:
-        raise yakonfig.ConfigurationError(
-            '{} requires top-level configuration for {}'
+        raise ConfigurationError(
+            '{0} requires top-level configuration for {1}'
             .format(who, config_name))
     checker = getattr(what, 'check_config', None)
     if checker:
         checker(config[config_name], config_name)
 
+
 def _recurse_config(parent_config, modules, f, prefix=''):
     '''Walk through the module tree.
 
@@ -265,18 +282,24 @@ def _recurse_config(parent_config, modules, f, prefix=''):
     for module in modules:
         config_name = getattr(module, 'config_name', None)
         if config_name is None:
-            raise yakonfig.ProgrammerError('{!r} must provide a config_name'
-                                           .format(module))
+            raise ProgrammerError('{0!r} must provide a config_name'
+                                  .format(module))
         new_name = prefix + config_name
 
         f(parent_config, config_name, new_name, module)
 
-        _recurse_config((parent_config or {}).get(config_name, None),
-                        getattr(module, 'sub_modules', []),
-                        f,
-                        new_name + '.')
+        try:
+            _recurse_config((parent_config or {}).get(config_name, None),
+                            getattr(module, 'sub_modules', []),
+                            f,
+                            new_name + '.')
+        except:
+            # achieve a sort of stack trace on the way out
+            logger.error('exception in _recurse_config of %s', module)
+            raise
     return parent_config
 
+
 def create_config_tree(config, modules, prefix=''):
     '''Cause every possible configuration sub-dictionary to exist.
 
@@ -300,8 +323,8 @@ def create_config_tree(config, modules, prefix=''):
             # this is the usual, expected case
             parent_config[config_name] = {}
         elif not isinstance(parent_config[config_name], collections.Mapping):
-            raise yakonfig.ConfigurationError(
-                '{} must be an object configuration'.format(prefix))
+            raise ConfigurationError(
+                '{0} must be an object configuration'.format(prefix))
         else:
             # config_name is a pre-existing dictionary in parent_config
             pass
@@ -329,17 +352,18 @@ def _walk_config(config, modules, f, prefix=''):
         # create_config_tree() needs to have been called by now
         # and you should never hit either of these asserts
         if config_name not in parent_config:
-            raise yakonfig.ProgrammerError('{} not present in configuration'
-                                           .format(prefix))
+            raise ProgrammerError('{0} not present in configuration'
+                                  .format(prefix))
         if not isinstance(parent_config[config_name], collections.Mapping):
-            raise yakonfig.ConfigurationError(
-                '{} must be an object configuration'.format(prefix))
+            raise ConfigurationError(
+                '{0} must be an object configuration'.format(prefix))
 
         # do the work!
         f(parent_config[config_name], module, prefix)
 
     return _recurse_config(config, modules, work_in)
 
+
 def collect_add_argparse(parser, modules):
     """Add all command-line options.
 
@@ -361,6 +385,7 @@ def collect_add_argparse(parser, modules):
     _recurse_config(dict(), modules, work_in)
     return parser
 
+
 def assemble_default_config(modules):
 
     """Build the default configuration from a set of modules.
@@ -376,12 +401,16 @@ def assemble_default_config(modules):
 
     """
     def work_in(parent_config, config_name, prefix, module):
+        my_config = dict(getattr(module, 'default_config', {}))
         if config_name in parent_config:
-            raise yakonfig.ProgrammerError('multiple modules providing {}'
-                                           .format(prefix))
-        parent_config[config_name] = dict(getattr(module, 'default_config', {}))
+            extra_config = parent_config[config_name]
+            raise ProgrammerError(
+                'config for {0} already present when about to fetch {3}.default_config (had {1!r} would have set {2!r})'.format(
+                    prefix, extra_config, my_config, module))
+        parent_config[config_name] = my_config
     return _recurse_config(dict(), modules, work_in)
 
+
 def fill_in_arguments(config, modules, args):
     """Fill in configuration fields from command-line arguments.
 
@@ -403,7 +432,7 @@ def fill_in_arguments(config, modules, args):
     """
     def work_in(config, module, name):
         rkeys = getattr(module, 'runtime_keys', {})
-        for (attr, cname) in rkeys.iteritems():
+        for (attr, cname) in iteritems(rkeys):
             v = args.get(attr, None)
             if v is not None:
                 config[cname] = v
@@ -411,6 +440,7 @@ def fill_in_arguments(config, modules, args):
         args = vars(args)
     return _walk_config(config, modules, work_in)
 
+
 def do_config_discovery(config, modules):
     '''Let modules detect additional configuration values.
 
@@ -428,9 +458,11 @@ def do_config_discovery(config, modules):
     '''
     def work_in(config, module, name):
         f = getattr(module, 'discover_config', None)
-        if f: f(config, name)
+        if f:
+            f(config, name)
     return _walk_config(config, modules, work_in)
 
+
 def normalize_config(config, modules):
     """Normalize configuration values in the entire tree.
 
@@ -448,5 +480,6 @@ def normalize_config(config, modules):
     """
     def work_in(config, module, name):
         f = getattr(module, 'normalize_config', None)
-        if f: f(config)
+        if f:
+            f(config)
     return _walk_config(config, modules, work_in)
